( cd usr/lib ; rm -rf libtag.so.1 )
( cd usr/lib ; ln -sf libtag.so.1.4.0 libtag.so.1 )
( cd usr/lib ; rm -rf libtag.so )
( cd usr/lib ; ln -sf libtag.so.1.4.0 libtag.so )
( cd usr/lib ; rm -rf libtag_c.so.0 )
( cd usr/lib ; ln -sf libtag_c.so.0.0.0 libtag_c.so.0 )
( cd usr/lib ; rm -rf libtag_c.so )
( cd usr/lib ; ln -sf libtag_c.so.0.0.0 libtag_c.so )
